#!/bin/bash
#
# Ex.2023/10/13
# 此Script功能說明
# 01.SQL產生備份前取得 /backup 剩餘空間，扣除全部資料庫大小，如果小於20G就不產生備份，並寄信通知
# 02.SQL產生備份紀錄於error log
# 03.error log 如果有錯誤訊息，將寄信通知，這樣可提早發現問題並修復
# 04.不備份 information_schema、performance_schema、sys

email='support@twnoc.net'

# 刪除昨日備份
rm -rf /backup/*-db-backup

local_disk_free=`df /backup | grep -v Filesystem | awk '{print $4}'`
mysql_size=`du -s /var/lib/mysql | awk '{print $1}'`
free=`expr $local_disk_free - $mysql_size`

# 剩餘空間少於20G不執行備份，寄信通知Jacky，空間夠則正常執行mysqldump
if (("$free" < "20971520")); then
    echo `hostname`'空間不足20G，無法正常執行SQL備份' | mutt -s `hostname`"無法正常執行SQL備份" $email
else

mkdir /backup/`date +%Y%m%d`'-db-backup'

/usr/bin/mysqlshow --defaults-file=/root/.my.cnf | awk '{print $2}' | awk 'NF' | sed -e '/Databases/d' | sed -e '/information_schema/d' | sed -e '/performance_schema/d' | sed -e '/sys/d' > /backup/dblist
exec < /backup/dblist

while read line
do
echo ''
echo "mysqldump is $line"
/usr/bin/mysqldump --defaults-file=/root/.my.cnf --max_allowed_packet=512M --databases --single-transaction --quick $line > /backup/`date +%Y%m%d`'-db-backup'/$line'.sql'
echo ''
done

fi

# 確認 error.log 是否有mysqldump錯誤，如果有將寄信通知Jacky
if grep -q Error /backup/sqldump_error.log; then
  grep Error -B1 /backup/sqldump_error.log | mutt -s `hostname`"執行mysqldump發生Error" $email
fi

